@@ -1,21 +1,87 @@
 /// Controller methods and structures
 use eva_common::actions::{self, ACTION_TOPIC};
-use eva_common::events::RAW_STATE_TOPIC;
+use eva_common::events::{RawStateEventOwned, RAW_STATE_TOPIC};
 use eva_common::prelude::*;
 use lazy_static::lazy_static;
 use serde::Deserialize;
+use std::collections::HashMap;
 use std::sync::Mutex;
 use std::time::{Duration, Instant};
+use ttl_cache::TtlCache;
 use uuid::Uuid;
 
 lazy_static! {
     static ref NEED_PANIC: Mutex<Option<Duration>> = <_>::default();
 }
 
+const RAW_STATE_CACHE_MAX_CAPACITY: usize = 1_000_000;
+
+pub struct RawStateCache {
+    cache: Mutex<TtlCache<OID, RawStateEventOwned>>,
+    ttl: Option<Duration>,
+}
+
+impl RawStateCache {
+    pub fn new(ttl: Option<Duration>) -> Self {
+        Self {
+            cache: Mutex::new(TtlCache::new(RAW_STATE_CACHE_MAX_CAPACITY)),
+            ttl,
+        }
+    }
+    /// returns true if no ttl set or the state is modified, false if the event is not required to
+    /// be sent
+    ///
+    /// # Panics
+    ///
+    /// Will panic if the mutex is poisoned
+    pub fn push_check(&self, oid: &OID, raw_state: &RawStateEventOwned) -> bool {
+        if let Some(ttl) = self.ttl {
+            let mut cache = self.cache.lock().unwrap();
+            if let Some(v) = cache.get(oid) {
+                if raw_state == v {
+                    return false;
+                }
+            }
+            cache.insert(oid.clone(), raw_state.clone(), ttl);
+            true
+        } else {
+            false
+        }
+    }
+    /// Removes from the state hashmap, these who are not required to be sent, caches
+    /// remaining
+    ///
+    /// # Panics
+    ///
+    /// Will panic if the mutex is poisoned
+    pub fn retain_map_modified(&self, states: &mut HashMap<&OID, RawStateEventOwned>) {
+        if let Some(ttl) = self.ttl {
+            let mut cache = self.cache.lock().unwrap();
+            states.retain(|oid, raw| {
+                if let Some(cached) = cache.get(oid) {
+                    cached != raw
+                } else {
+                    true
+                }
+            });
+            // cache kept ones
+            for (oid, raw) in states {
+                cache.insert((*oid).clone(), raw.clone(), ttl);
+            }
+        }
+    }
+}
+
+/// # Panics
+///
+/// Will panic if the mutex is poisoned
 pub fn set_poc(panic_in: Option<Duration>) {
     *NEED_PANIC.lock().unwrap() = panic_in;
 }
 
+/// # Panics
+///
+/// Will panic if the mutex is poisoned
 pub fn poc() {
     if let Some(delay) = *NEED_PANIC.lock().unwrap() {
         bmart::process::suicide(delay + Duration::from_secs(1), false);
